/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006, 2007, 2008  Klaas van Gend, Christofer Wesseling
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include "umtsmon_global.h"
#include "DeviceCapabilities.h"
#include "TheDeviceManagerSingleton.h"
#include "PPPConnection.h"
#include "Popup.h"

#include "radiopreferences_defines.h"

#include "qlabel.h"
#include "qlineedit.h"
#include "qtextedit.h"

#include <assert.h>

// time between two refreshes in seconds
#define SEC_REFRESH   3



/** this class contains the observer for the PPPConnection
 *  it is owned and contained by mainwindow
 */
class MainWindowPPPObserver : public PPPObserverInterface
{
public:
	/// constructor
	MainWindowPPPObserver(mainwindow* aMainWindowPtr) : theMainWindowPtr(aMainWindowPtr)  {;};	
	virtual void newPPPState(PPPDState aNewState)
		{
			DEBUG2("MainWindowObserver::newPPPState %d\n", aNewState);
			theMainWindowPtr->newPPPState(aNewState);
		};			
	
private:
	mainwindow* theMainWindowPtr;
};


void mainwindow::init()
{
	myRefreshTimer = NULL;
	
	setCaption("UMTSmon");
	menu_help_about->setMenuText("&About " APPNAME );
	menu_connection_exit->setMenuText("&Quit " APPNAME );
	Toolbar->setCaption(APPNAME" Toolbar");
	
	// lets check the PIN-Status... so we can make the menu user friendly... :)
	SIMHandler mySIMHandler;
	if(!mySIMHandler.isPINProtectionEnabled())
	{
		DEBUG5(" PIN protection on SIM is disabled \n");		    
		//let the user see whats active and not...
		menu_pin_enable->setOn( false );	
		menu_pin_enable->setEnabled(true);
		
		menu_pin_disable->setOn( true );
		menu_pin_disable->setEnabled(false); 
		
		menu_pin_change->setEnabled(false);
	}
	else
	{
		DEBUG5(" PIN protection on SIM is enabled \n");
		//let the user see whats active and not...
		menu_pin_enable->setOn( true );	
		menu_pin_enable->setEnabled(false);
		
		menu_pin_disable->setOn( false );
		menu_pin_disable->setEnabled(true); 
		
		menu_pin_change->setEnabled(true);	       
	}
	
	emit setNetworkAutoUpdate();
	emit updateNetworkData();

	theMainWindowPPPObserverPtr = new MainWindowPPPObserver(this);
	thePPPConnection.attach(theMainWindowPPPObserverPtr);
	
}


void mainwindow::showAboutDialog()
{
	DEBUG5("About dialog clicked\n");
	AboutDialogUmtsmon myDialog;
	myDialog.versionlabel->setText("<b>"APPNAME"</b> version <b>"APPRELEASE "" APPFLAVOUR "</b>");
	// no AT commands in this dialog - no exec() prevention needed
	myDialog.exec();
}

void mainwindow::showInfoDialog()
{
	DEBUG5("info dialog clicked\n");
	if (ifATPortAvailableRunExec(NULL))
	{	
		InfoDialogUmtsmon myDialog;
		myDialog.lbl_pppd_version->setText(thePPPConnection.getPPPVersion());
		ifATPortAvailableRunExec(&myDialog);
	}
}



void mainwindow::DisablePINProtection()
{
    SIMHandler mySIMHandler;
    	EnterPIN myPIN;
			myPIN.enableValidator();					
			myPIN.setSIMHandlerPtr(&mySIMHandler);							
			myPIN.exec(); // shows the dialog, blocking all other..
			if(myPIN.result() == QDialog::Rejected) return; 
    if(!mySIMHandler.setPINActive(false,myPIN.pinField->text(),this))
	    	Popup::Warning(tr("Disabling PIN protection failed.\n"
                    "Try again, and be sure to use the correct current PIN!"), this);
    else
    {
	 //switch the toggle state....
	
	 //let the user see whats active and not...
	menu_pin_enable->setOn( false );	
	menu_pin_enable->setEnabled(true);
	
	menu_pin_disable->setOn( true );
	menu_pin_disable->setEnabled(false); 
	
	menu_pin_change->setEnabled(false);
	Popup::Info("Disabling PIN protection was successfull.");

    }
    
}


void mainwindow::EnablePINProtection()
{
    SIMHandler mySIMHandler;
        	EnterPIN myPIN;
			myPIN.enableValidator();					
			myPIN.setSIMHandlerPtr(&mySIMHandler);							 
			myPIN.exec(); // shows the dialog, blocking all other..
		if(myPIN.result() == QDialog::Rejected) return;
    if(!mySIMHandler.setPINActive(true,myPIN.pinField->text(),this))
		Popup::Warning(tr("Enabling PIN protection failed.\n"
                    "Try again, and be sure to use the correct current PIN!"), this);
	 else
    {
	 //let the user see whats active and not...
	menu_pin_enable->setOn( true );	
	menu_pin_enable->setEnabled(false);
	
	menu_pin_disable->setOn( false );
	menu_pin_disable->setEnabled(true); 
	
	menu_pin_change->setEnabled(true);
	Popup::Info(tr("PIN protection was successfull."), this);

    }
}


void mainwindow::ChangePIN()
{
    DEBUG5("change PIN settings clicked\n");
    SIMHandler mySIMHandler;
          EnterPIN myCurrentPIN;
			myCurrentPIN.enableValidator();					
			myCurrentPIN.setSIMHandlerPtr(&mySIMHandler);							
			myCurrentPIN.errorMessage->setText(tr("Please enter your current PIN.")); 
			myCurrentPIN.exec(); // shows the dialog, blocking all other..	
		if(myCurrentPIN.result() == QDialog::Rejected) return;
			
		EnterPIN myNewPIN;
			myNewPIN.enableValidator();					
			myNewPIN.setSIMHandlerPtr(&mySIMHandler);							
			myNewPIN.errorMessage->setText(tr("Please enter your NEW pin.")); 
			myNewPIN.exec(); // shows the dialog, blocking all other..
		if(myNewPIN.result() == QDialog::Rejected) return;
		
		EnterPIN myNewPINsec;
			myNewPINsec.enableValidator();					
			myNewPINsec.setSIMHandlerPtr(&mySIMHandler);							
			myNewPINsec.errorMessage->setText(tr("Please verify your NEW pin!")); 
			myNewPINsec.exec(); // shows the dialog, blocking all other..
		if(myNewPINsec.result() == QDialog::Rejected) return;

	if(!myNewPINsec.pinField->text().contains(myNewPIN.pinField->text() ) )
	{
		Popup::Warning(tr("Changing PIN failed.\n"
		    "New PIN and verify PIN were not same, please try again!"), this);
	}
	else if(! mySIMHandler.setNewPIN(myCurrentPIN.pinField->text(),myNewPIN.pinField->text(),this))  
	{
		Popup::Warning(tr("Changing PIN failed.\n"
		    "Try again, and be sure to use the correct current PIN!"), this);
	}
	else
	{
		Popup::Info(tr("PIN changed successfully."), this);
	}
}


void mainwindow::PushButtonCreateSMS_clicked()
{
	DEBUG5("mainwindow::PushButtonCreateSMS_clicked()\n");	
	if (checkRegistered()==true)
	{
		smsdialog myDialog;
		ifATPortAvailableRunExec(&myDialog);
	}
	else
	{
		Popup::Warning(tr("You are not connected to a network yet.\n"
		  "SMS functions are unavailable."), this);
	}
}


void mainwindow::showPPPErrorLogsDialog()
{
	PPPErrorLogsDialog myPPPDialog;
	QStringList::Iterator it;
	if (thePPPConnection.thePPPDStdOut.count() > 0)
	{
		for (it = thePPPConnection.thePPPDStdOut.begin(); 
		     it != thePPPConnection.thePPPDStdOut.end(); ++it )
		{
			myPPPDialog.StdOutTextBox->append(*it);
		}
	}
	else
		myPPPDialog.StdOutTextBox->append(
			tr("<<PPP did not provide any stdout information>>"));

	if (thePPPConnection.thePPPDStdErr.count() > 0)
	{
		for (it = thePPPConnection.thePPPDStdErr.begin(); 
		     it != thePPPConnection.thePPPDStdErr.end(); ++it )
		{
			myPPPDialog.StdErrTextBox->append(*it);
		}
	}
	else
		myPPPDialog.StdErrTextBox->append(
			tr("<<PPP did not provide any stderr information>>"));

	myPPPDialog.exec();	
}


void mainwindow::updateNetworkData()
{
	DEBUG5("mainwindow::updateNetworkData()\n");
	ConnectionInfo* myCIPtr = TheDeviceManagerSingleton::getConnectionInfoPtr();
	if (myCIPtr == NULL)
		return;

	// there's three situations here:
	// 1) minimized - display in title bar
	// 2) maximized, in network tab - update fields inside that tab
	// 3) maximized, not in network - do nothing
	
	thePPPConnection.isPPPConnected();

	try { myCIPtr->refresh(); }
	catch (MonthRolloverException)
	{
		Popup::Warning("Your computer just ran into a new month.\n"
					"Due to limitations of pppd, we cannot account new"
					"traffic to the new month. Please close the connection"
					"and reconnect at your earliest convenience.");
	}
	catch (OverThresholdException)
	{
		Popup::Warning("You are over your data limit treshold.");
	}
	
	// and update title bar or fields with data
	if (isMinimized())
	{
		DEBUG5("umtsmon minimized - change titlebar\n");
		QString myRT = myCIPtr->getRadiotype();
		QString myTotalName = APPNAME " " + myRT + " " + myCIPtr->getSignalQualityString(); 
		setCaption( myTotalName );
	}
	else
	{
		setCaption("UMTSmon");
		// only the last one needs to have the LCDDisplay redraw
		theLCDDisplay->setStrength(myCIPtr->getSignalQuality(), false);
		theLCDDisplay->setRoaming( myCIPtr->isRoaming());
		theLCDDisplay->setOperatorName( myCIPtr->getOperatorName(), false);
		theLCDDisplay->setRadioType( myCIPtr->getRadiotype(), false );
		theLCDDisplay->setUpload(myCIPtr->getCounterBytesSent(), 
								 myCIPtr->getUploadDelta() / SEC_REFRESH, false);
		theLCDDisplay->setDownload(myCIPtr->getCounterBytesReceived(),
								 myCIPtr->getDownloadDelta() / SEC_REFRESH, false);
		theLCDDisplay->setConnectedTime(myCIPtr->getSecondsConnected(), false);

		// alternate in the display between the total time and total bytes
		static int theVisualCounter = 0;
		theVisualCounter++;
		if (theVisualCounter < 3)
			theLCDDisplay->setTotalTraffic(
							myCIPtr->getTotalTraffic(), 
							myCIPtr->getTrafficThresholdStatus(), true);
		else
		{
			theLCDDisplay->setTotalTime(
							myCIPtr->getTotalSecondsConnected(), 
							myCIPtr->getTimeThresholdStatus(), true);
			if (theVisualCounter >4)
				theVisualCounter = 0;
		}
		
		
	}
}





void mainwindow::setNetworkAutoUpdate()
{
	DEBUG5("setNetworkAutoUpdate slot\n");
	if (TheSettingsSingleton::getQSRef().readBoolEntry(
	  THE_RADIO_PREF_ROOT_NAME THE_RADIOREFRESH_SETTING, true))
	{
		if (myRefreshTimer == NULL)
		{
			// setup the refreshtimer for network statistics and signal quality
			myRefreshTimer = new QTimer(this, "refreshtimer");
			connect(myRefreshTimer, SIGNAL(timeout()), SLOT(updateNetworkData()));
			updateNetworkData();
			myRefreshTimer->start(SEC_REFRESH * 1000, FALSE);
		}
		else
		{
			DEBUG1("refreshtimer not null?!?");
		}
		// no need for the refresh button if we're on automatic update
		toolbar_button_refresh->setEnabled(false);
	}
	else
	{
		if (myRefreshTimer != NULL)
		{
			myRefreshTimer->stop();
			delete myRefreshTimer;
			myRefreshTimer = NULL;
		}
		// user might need the refresh button now
		toolbar_button_refresh->setEnabled(true);
	}
	// whether we set a timer or just killed it - let's refresh now
	emit updateNetworkData();
}

// only called from class MainWindowObserver
void mainwindow::newPPPState(PPPObserverInterface::PPPDState aNewState)
{
	DEBUG3("mainwindow::newPPPState()\n");
	switch(aNewState)
	{
	case PPPObserverInterface::ERROR:
		emit showPPPErrorLogsDialog();
		emit updateNetworkData();
		break;
	case PPPObserverInterface::STOPPED:
		// there is no connection
		menu_connection_connect->setOn( false );	
		menu_connection_disconnect->setOn( true );
		// so the 'disconnect' buttons are disabled anyway
		menu_connection_disconnect->setEnabled( false );
		toolbar_button_disconnect->setEnabled( false );
		// can we start a new connection?
		if (TheDeviceManagerSingleton::getDeviceCapsRef().getPPPSerialPortName().isEmpty() == false)
		{
			menu_connection_connect->setEnabled( true );
			toolbar_button_connect->setEnabled( true );
			break;
		}
		// if we are still here, we cannot
		menu_connection_connect->setEnabled( false );
		toolbar_button_connect->setEnabled( false );
		break;
	case PPPObserverInterface::STARTING:
	case PPPObserverInterface::NOT_OURS_RUNNING:
		// there *is* a connection
		menu_connection_connect->setOn( true );	
		menu_connection_disconnect->setOn( false );
		// so the 'connect' buttons are disabled anyway
		menu_connection_connect->setEnabled( false );
		toolbar_button_connect->setEnabled( false );
		// and the PPPD is not (yet) ours - so we shouldn't disable it
		menu_connection_disconnect->setEnabled( false );
		toolbar_button_disconnect->setEnabled( false );
		break;
		break;
	case PPPObserverInterface::RUNNING:
		// there *is* a connection
		menu_connection_connect->setOn( true );	
		menu_connection_disconnect->setOn( false );
		// so the 'connect' buttons are disabled anyway
		menu_connection_connect->setEnabled( false );
		toolbar_button_connect->setEnabled( false );
		// and the PPPD is ours - so we are allowed to disable it
		menu_connection_disconnect->setEnabled( true );
		toolbar_button_disconnect->setEnabled( true );
		break;
	}
}

bool mainwindow::DisconnectPPP()
{
	DEBUG4("umtsmonmain::DisconnectPPP\n");
	thePPPConnection.stopPPP();
	emit updateNetworkData();
	return true;
}

bool mainwindow::checkRegistered()
{
	// ask a ConnectionInfo if we are operator connected already
	ConnectionInfo* myCIPtr = TheDeviceManagerSingleton::getConnectionInfoPtr();
	assert(myCIPtr != NULL);
	
	// none of the exceptions can be thrown here as we're not connected 
	myCIPtr->refresh();
	
	if (myCIPtr->isSearching()==true)
	{
		Popup::Info("There is no connection with a mobile network yet.\n"
				"Please wait a few more seconds. If the problem remains,"
				"try selecting another network operator.");
		return false;
	}
	if (!myCIPtr->isDeviceRegistered())
	{
		Popup::Warning(tr("You are not connected to a network yet."), this);
		return false;
	}

	return true;
}


bool mainwindow::ConnectPPP()
{
	DEBUG3("umtsmonmain::ConnectPPP\n");

	// check if we are registered on a network already
	if (checkRegistered()==false)
		return false;
	// we are - let's start the PPP!
	// startPPP doesn't block for more than 6 seconds 
	//   - so we can do relatively adequate progress reports
	ProgressDialog myProgress(this, tr("Setting up PPP Connection"));
	myProgress.setMaxTime(30);
	// connection will take time anyway, so better show progress from start
	myProgress.start(0);

	if(thePPPConnection.startPPP()==false)
		goto problems;
	DEBUG2("thePPPConnection.startPPP() returned without indicating problems\n");

	while (thePPPConnection.isPPPStarting())
	{
		if (myProgress.runOneSecond() == false)
		{
			// Apparently, the user pressed cancel?!
			goto cleanup;
		}
	}
	myProgress.finish();
	if (thePPPConnection.hasErrors())
		goto problems;
	emit updateNetworkData();

	return true;
	
	//---------------------
problems:	
	DEBUG1("start PPP NOT successful :-(\n");
// FIXME: Do we still need this warning message?
	Popup::Warning(tr("Failed to start PPP.\n"
	       "Please re-run umtsmon from the commandline with -v5 and check the output."), this);
	
cleanup:
	sleep(1);
	DisconnectPPP();
	emit updateNetworkData();
	return false;
}


void mainwindow::showProfileDialog()
{
	DEBUG5(" profile dialog button clicked\n");	
	ManageProfiles myDialog;
	ifATPortAvailableRunExec( &myDialog );
}



void mainwindow::showNetworkOperatorDialog()
{
	DEBUG5(" network operator button clicked\n");

	// Only allow switching of operator if no connection active
	NetworkOperatorWizard myDialog;
	if (ifPPPInactiveRunExec(&myDialog)==true)
	{
		if(myDialog.result() == QDialog::Accepted)
			emit updateNetworkData();
	}
}


/** executes the dialog (i.e. aDialogToExecPtr->exec()) 
 *    if either of these conditions is met:
 *      a) the card has more than one serial port
 *      b) there is no running PPP session.
 *    Otherwise it will throw a warning dialog.
 *  @param aDialogToExecPtr  QDialog subclass to run exec() on
 *  @returns  true if dialog was executed, false if not
 */
bool mainwindow::ifATPortAvailableRunExec( QDialog* aDialogToExecPtr)
{
	DEBUG5("mainwindow::ifATPortAvailableRunExec()\n");
	// test for condition A)
	if (TheDeviceManagerSingleton::getDeviceCapsRef().isSinglePort()==false)
	{
		if (aDialogToExecPtr!=NULL)	
			aDialogToExecPtr->exec();
		return true;
	}

	// test for condition B)
	DEBUG5("only one port available - check if PPP is running?\n");
	return ifPPPInactiveRunExec(aDialogToExecPtr);
}

/** executes the dialog (i.e. aDialogToExecPtr->exec()) 
 *    IF AND ONLY IF there is no running PPP session.
 *    Otherwise it will throw a warning dialog.
 *  @param aDialogToExecPtr  QDialog subclass to run exec() on
 *  @returns  true if dialog was executed, false if not
 */
bool mainwindow::ifPPPInactiveRunExec(QDialog* aDialogToExecPtr)
{
	DEBUG5("mainwindow::ifPPPInactiveRunExec()\n");
	// hack - test for the start PPP icon to be enabled
	if (menu_connection_connect->isEnabled())
	{
		ConnectionInfoInhibitor myParallelATPreventer;
		if (aDialogToExecPtr!=NULL)	
			aDialogToExecPtr->exec();
		return true;
	}
	Popup::Warning(tr("PPP is active, cannot run this in parallel."), this);
	return false;
}



void mainwindow::showRadioPreferences()
{
	DEBUG1("   showRadioPreferences dialog requested\n");

	//  this window is modeless - one can keep it open
	RadioPreferences myDialog;
	myDialog.connect (&myDialog, SIGNAL(autoUpdateChanged()), this, SLOT(setNetworkAutoUpdate())); 
	myDialog.exec();
	myDialog.disconnect();
}
